/* Compile with 
 *   gcc test.c `pkg-config libgda-4.0 --libs --cflags`
 */

#include <libgda/libgda.h>
#include <sql-parser/gda-sql-parser.h>
#include <string.h>

int
main (int argc, char **argv)
{
	gda_init ();

	GError* error = NULL;
	GdaConnection* cnc = gda_connection_open_from_string ("PostgreSQL", "DB_NAME=sales", "", (GdaConnectionOptions)0, &error);
	if(error) {
		g_print ("GError from gda_connection_open_connection_from_string(): %s\n", error->message);
		g_clear_error (&error);
		return EXIT_FAILURE;
	}

	/* update tables list */
	const gchar *table_name;
	if (argc != 2) {
		g_print ("Usage: %s <table_name>\n", argv[0]);
		return EXIT_FAILURE;
	}

	table_name = argv[1];
	g_print ("Updating the meta store for the '%s' table\n", table_name);

	GdaMetaContext mcontext = {"_tables", 1, NULL, NULL};
	mcontext.column_names = g_new (gchar *, 1);
	mcontext.column_names[0] = "table_name";
	mcontext.column_values = g_new (GValue *, 1);
	g_value_set_string ((mcontext.column_values[0] = gda_value_new (G_TYPE_STRING)), argv[1]);

	gboolean upd_res;
	upd_res = gda_connection_update_meta_store (cnc, &mcontext, &error);
	if (! upd_res) {
		g_print ("GError from gda_connection_update_meta_store(): %s\n", error->message);
		g_clear_error (&error);
		return EXIT_FAILURE;
	}

	GdaDataModel *model;
	model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), "SELECT * FROM _tables", &error, NULL);
	if (!model) {
		g_print ("GError from gda_meta_store_extract(): %s\n", error->message);
		g_clear_error (&error);
		return EXIT_FAILURE;
	}
	g_print ("Number of tables: %d\n", gda_data_model_get_n_rows (model));
	g_object_unref (model);

	model = gda_meta_store_extract (gda_connection_get_meta_store (cnc), "SELECT table_catalog, table_schema, table_name, column_name FROM _columns ORDER BY ordinal_position", &error, NULL);
	if (!model) {
		g_print ("GError from gda_meta_store_extract(): %s\n", error->message);
		g_clear_error (&error);
		return EXIT_FAILURE;
	}
	g_print ("Columns: %d\n", gda_data_model_get_n_rows (model));
	gda_data_model_dump (model, NULL);
	g_object_unref (model);

	g_object_unref (cnc);

	return EXIT_SUCCESS;
}
